﻿using Quartz;
using SqlSugar;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Hotline.CallCenter.Calls;
using Hotline.Repository.SqlSugar;
using Hotline.Share.Enums.CallCenter;
using Hotline.Users;
using MapsterMapper;
using Microsoft.Extensions.Logging;
using XF.Domain.Cache;
using XF.Domain.Repository;
using XingTang.Sdk;

namespace Hotline.Application.Jobs
{
    /// <summary>
    /// 查询通话记录
    /// </summary>
    public class GetCallsJob : IJob, IDisposable
    {
        private readonly IRepository<TrCallRecord> _trcallRepository;
        private readonly IRepository<User> _userRepository;
        private readonly IMapper _mapper;
        private readonly ILogger<GetCallsJob> _logger;
        private readonly ISqlSugarClient _db;

        public GetCallsJob(
            ISugarUnitOfWork<XingTangDbContext> uow,
            IRepository<TrCallRecord> trcallRepository,
            IRepository<User> userRepository,
            IMapper mapper,
            ILogger<GetCallsJob> logger)
        {
            _trcallRepository = trcallRepository;
            _userRepository = userRepository;
            _mapper = mapper;
            _logger = logger;
            _db = uow.Db;
        }

        public async Task Execute(IJobExecutionContext context)
        {
            var calls = await _db.Queryable<XingtangCall>()
                .Where(d => (d.IsSync == null || !d.IsSync) && (d.Tries == null || d.Tries <= 50))
                .Take(100)
                .ToListAsync(context.CancellationToken);

            var occupyCalls = new List<XingtangCall>();
            foreach (var call in calls)
            {
                var rows = await _db.Updateable(new XingtangCall
                {
                    Id = call.Id,
                    IsSync = true,
                    Tries = call.Tries + 1
                }).ExecuteCommandWithOptLockAsync();
                if (rows > 0)
                    occupyCalls.Add(call);
            }

            try
            {
                var trCalls = _mapper.Map<List<TrCallRecord>>(occupyCalls);
                //填充user信息
                var staffNos = calls.Select(d => d.UserCode).ToList();
                var users = await _userRepository.Queryable()
                    .Where(d => staffNos.Contains(d.StaffNo))
                    .ToListAsync(context.CancellationToken);

                foreach (var trCall in trCalls)
                {
                    //if (trCall.CallDirection is ECallDirection.Out &&
                    //    string.IsNullOrEmpty(trCall.RecordingAbsolutePath))
                    //    continue;
                    Console.WriteLine(trCall.OtherAccept);
                    
                    var user = users.FirstOrDefault(d => d.StaffNo == trCall.StaffNo);
                    if (user is not null)
                    {
                        trCall.UserId = user.Id;
                        trCall.UserName = user.Name;
                    }
                }

                await _trcallRepository.UpdateRangeAsync(trCalls, context.CancellationToken);
            }
            catch (Exception e)
            {
                //Console.WriteLine(e);
                _logger.LogError($"获取通话记录异常：{e.Message} \n {e.StackTrace}");
                foreach (var occupyCall in occupyCalls)
                {
                    occupyCall.IsSync = false;
                }

                await _db.Updateable(occupyCalls)
                    .UpdateColumns(d => new { d.IsSync })
                    .ExecuteCommandAsync(context.CancellationToken);
            }
        }

        /// <summary>Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.</summary>
        public void Dispose()
        {
            _logger.LogInformation($"{nameof(GetCallsJob)} disposed");
        }
    }
}